#include <stdio.h>
#include <jpeglib.h>
#include <netinet/in.h>
#include "opbcam-grab.h"

int tcp_dest_error;

static struct {
	unsigned long len; // use htonl()
	unsigned char data[MAX_FRAME];
} jpgBuf;

static const int TARGET_LO = (int)((float)TARGET_SIZE * 0.95);
static const int TARGET_HI = (int)((float)TARGET_SIZE * 1.05);

static struct jpeg_destination_mgr my_dest;
static int quality;
static int overflow;
static int sockfd;



static void init_tcp_destination(j_compress_ptr cinfo)
{
	jpeg_set_quality(cinfo, quality, TRUE);
	my_dest.next_output_byte = jpgBuf.data;
	my_dest.free_in_buffer = MAX_FRAME;
	overflow = 0;
	jpgBuf.len = 0;
}

static boolean empty_tcp_output_buffer(j_compress_ptr cinfo)
{
	my_dest.next_output_byte = jpgBuf.data;
	my_dest.free_in_buffer = MAX_FRAME;
	overflow = 1;
	return TRUE;
}

static void term_tcp_destination(j_compress_ptr cinfo)
{
	int imgSize = overflow ? MAX_FRAME :
		my_dest.next_output_byte - jpgBuf.data;

	//printf("Jpeg Done %d q%d\n", imgSize, quality);

	if (imgSize < TARGET_LO)
	{
		quality++;
		if (quality > MAX_QUALITY) quality = MAX_QUALITY;
	}
	else if (imgSize > TARGET_HI)
	{
		quality--;
		if (quality < MIN_QUALITY) quality = MIN_QUALITY;
	}

	tcp_dest_error = 1;

	if (imgSize < MAX_FRAME)
	{
		jpgBuf.len = htonl(imgSize);
		if (sendall(sockfd, (char *)&jpgBuf, 4+imgSize) != -1)
		{
			tcp_dest_error = 0;
		}
	}
}

void tcp_dest(j_compress_ptr cinfo, int fd) 
{
	sockfd = fd;
	quality = MIN_QUALITY;
	my_dest.init_destination = init_tcp_destination;
	my_dest.empty_output_buffer = empty_tcp_output_buffer;
	my_dest.term_destination = term_tcp_destination;
	cinfo->dest = &my_dest;
}
