#include "opbcam-common.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <sys/mman.h>

struct flock index_lk, buf_lk[BUF_COUNT];
static pid_t pid;


void init_locks(void)
{
	int i;
	struct shm_data *s = 0;

	pid = getpid();

	make_lock(&index_lk, (off_t)&(s->index), sizeof(s->index));
	for (i=0; i<BUF_COUNT; i++)
	{
		make_lock(&buf_lk[i], (off_t)&(s->buf[i]), sizeof(s->buf[i]));
	}
}


int init_shm(struct shm_data **shm, int prot)
{
	int fd;

	if ((fd = shm_open(SHM_FILE, O_CREAT|O_RDWR, SHM_MODE)) == -1)
	{
		perror("shm_open");
		exit(1);
	}

	if (ftruncate(fd, sizeof(struct shm_data)) == -1)
	{
		perror("ftruncate");
		exit(1);
	}

	if ((*shm = (struct shm_data *)mmap(NULL, sizeof(struct shm_data),
			prot, MAP_SHARED, fd, 0)) == MAP_FAILED)
	{
		perror("mmap");
		exit(1);
	}

	return fd;
}


void make_lock(struct flock *fl, off_t base, size_t len)
{
	fl->l_whence = SEEK_SET;
	fl->l_start = base;
	fl->l_len = len;
	fl->l_pid = pid;
}


void lock_wait(int fd, short type, struct flock *fl)
{
	fl->l_type = type;

	if (fcntl(fd, F_SETLKW, fl) == -1)
	{
		perror("fcntl");
		exit(1);
	}
}


int initUDP(struct sockaddr_in *to_addr)
{
	int fd;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
	{
		perror("socket");
		exit(1);
	}

	to_addr->sin_family = AF_INET;
	to_addr->sin_port = htons(SERVER_PORT);
	to_addr->sin_addr.s_addr = inet_addr(SERVER_ADDR);
	memset(&(to_addr->sin_zero), 0, 8);

	return fd;
}


void sendUDP(int fd, char *data, int len, struct sockaddr_in *to)
{
	if (sendto(fd, data, len, 0, (struct sockaddr *)to,
				sizeof(struct sockaddr_in)) == -1)
	{
		perror("sendto");
		exit(1);
	}
}


void readall(void *buf, size_t count)
{
	int n;
	while (count > 0)
	{
		n = read(STDIN_FILENO, buf, count);
		if (n <= 0)
		{
			perror("read");
			exit(1);
		}
		buf = (char*)buf + n;
		count -= n;
		if (count > 0) usleep(1000);
	}
}


void writeall(void *buf, size_t count)
{
	int n;

	while (count > 0)
	{
		n = write(STDOUT_FILENO, buf, count);
		if (n <= 0)
		{
			perror("write");
			exit(1);
		}
		buf = (char*)buf + n;
		count -= n;
		if (count > 0) usleep(1000);
	}
}
