#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/mman.h>
#include <netinet/in.h>
#include "opbcam-common.h"

int lock_poll(int fd, int start, int len);
int count_viewers(int fd, int l, int r);
void update_count_file(int nViewers);
void check_source(void);

int main(void)
{
	int shm_fd, udp_fd;
	struct shm_data *shm;
	int bufi = 0; // write-buffer index
	int size_in;
	struct sockaddr_in to_addr;
	time_t now, last_ping = 0;

	int nViewers;


	printf("Content-type: text/plain\r\n\r\n");

	check_source();

	udp_fd = initUDP(&to_addr);

	init_locks();

	// Connect to shared memory
	shm_fd = init_shm(&shm, PROT_READ|PROT_WRITE);


	// Loop as long as there's at least 1 viewer
	while(lock_poll(shm_fd, 0, MAX_VIEWERS))
	{
		// Every 5 seconds, ping the server and update the count file
		now = time(NULL);
		if (now - last_ping > 5)
		{
			nViewers = count_viewers(shm_fd, 0, MAX_VIEWERS);
			
			if (nViewers == 0) {
				break;
			}

			sendUDP(udp_fd, "ON!", 3, &to_addr);
			last_ping = now;

			update_count_file(nViewers);
		}

		// Read a frame
		readall(&size_in, sizeof(size_in));
		size_in = ntohl(size_in);
		if (size_in <= 0 || size_in > MAX_FRAME)
		{
			fprintf(stderr, "Bad frame size\n");
			exit(1);
		}

		// Lock, update, unlock the buffer
		lock_wait(shm_fd, F_WRLCK, &buf_lk[bufi]);
		shm->buf[bufi].len = size_in;
		readall(&(shm->buf[bufi].frame), size_in);
		lock_wait(shm_fd, F_UNLCK, &buf_lk[bufi]);

		// Lock, update, unlock the index
		lock_wait(shm_fd, F_WRLCK, &index_lk);
		shm->index = bufi;
		lock_wait(shm_fd, F_UNLCK, &index_lk);

		// Increment sync to advertise the new frame 
		shm->sync++;

		// Move the write cursor
		bufi = (bufi+1) % BUF_COUNT;
	}

	update_count_file(0);

	// Send "OFF" packet to the server
	sendUDP(udp_fd, "OFF", 3, &to_addr);

	return 0;
}

void check_source()
{
	char *addr;
    addr = getenv("REMOTE_ADDR");

	if (addr == NULL)
	{
		printf("ERROR: can't get REMOTE_ADDR");
	}
	else if(strcmp(addr, SERVER_ADDR) != 0)
	{
		printf("ERROR: '%s' != '%s'", addr, SERVER_ADDR);
	}
	else
	{
		printf("'%s' OK.", addr);
		return;
	}

	exit(0);
}

// Returns true if write-locks exist in the specified range
int lock_poll(int fd, int start, int len)
{
	struct flock fl;
	make_lock(&fl, start, len);
	fl.l_type = F_RDLCK;
	if (fcntl(fd, F_GETLK, &fl) == -1)
	{
		perror("fcntl");
		exit(1);
	}
	return fl.l_type != F_UNLCK;
}

// recursively count viewers in the range [l,r)
int count_viewers(int fd, int l, int r)
{
	int m;
	if (l>=r) return 0;

	// If there aren't any viewers in range, return 0
	if (!lock_poll(fd, l, r-l))
	{
		return 0;
	}

	// If the range is 1, return 1 viewer.
	if (r-l == 1)
	{
		return 1;
	}

	// Otherwise, cut into halves and return sum
	m = (l+r)/2;
	return count_viewers(fd,l,m) + count_viewers(fd,m,r);
}

void update_count_file(int nViewers)
{
	// Store count in text file
	FILE *fp;

	if ((fp = fopen("count.txt", "w")) != NULL)
	{
		fprintf(fp, "<b>%d</b>\n", nViewers);
		fclose(fp);
	}
}
